﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace SharpGIS.Silverlight.Behaviors
{
	public class ToggleCollapseAction : System.Windows.Interactivity.TargetedTriggerAction<UIElement>
	{
		Storyboard uncollapseStoryboard, collapseStoryboard;

		protected override void OnTargetChanged(UIElement oldTarget, UIElement newTarget)
		{
			if(newTarget != null && !(newTarget.RenderTransform is ScaleTransform))
				newTarget.RenderTransform = new ScaleTransform(); //This could overwrite existing transforms. Beware!
			base.OnTargetChanged(oldTarget, newTarget);
		}

		protected override void Invoke(object parameter)
		{
			if (Target.Visibility == Visibility.Collapsed ||
				collapseStoryboard != null && collapseStoryboard.GetCurrentState() == ClockState.Active) //Uncollapse
			{
				if (uncollapseStoryboard == null)
					uncollapseStoryboard = CreateStoryboard(1, Visibility.Visible);
				if (collapseStoryboard != null)
					collapseStoryboard.SkipToFill();
				uncollapseStoryboard.Begin();
			}
			else //Collapse
			{
				if (collapseStoryboard == null)
					collapseStoryboard = CreateStoryboard(0, Visibility.Collapsed);
				if (uncollapseStoryboard != null)
					uncollapseStoryboard.SkipToFill();
				collapseStoryboard.Begin();
			}
		}

		private Storyboard CreateStoryboard(double to, Visibility state)
		{
			Storyboard s = new Storyboard();
			//ScaleX
			if (CollapseHorizontal || state == Visibility.Visible)
			{
				DoubleAnimation a1 = new DoubleAnimation() { To = to, From = 1 - to };
				if (CollapseHorizontal) a1.Duration = this.Duration;
				else if (state == Visibility.Visible) a1.Duration = TimeSpan.FromMilliseconds(0);
				a1.SetValue(Storyboard.TargetPropertyProperty, new PropertyPath("ScaleX"));
				Storyboard.SetTarget(a1, Target.RenderTransform);
				s.Children.Add(a1);
			}
			//ScaleY
			if (CollapseVertical || state == Visibility.Visible)
			{
				DoubleAnimation a2 = new DoubleAnimation() { To = to, From = 1 - to };
				if (CollapseVertical) a2.Duration = this.Duration;
				else if (state == Visibility.Visible) a2.Duration = TimeSpan.FromMilliseconds(0);
				a2.SetValue(Storyboard.TargetPropertyProperty, new PropertyPath("ScaleY"));
				Storyboard.SetTarget(a2, Target.RenderTransform);
				s.Children.Add(a2);
			}
			//Opacity
			if (CollapseOpacity || state == Visibility.Visible)
			{
				DoubleAnimation a3 = new DoubleAnimation() { To = to, From = 1 - to };
				if (CollapseOpacity) a3.Duration = this.Duration;
				else if (state == Visibility.Visible) a3.Duration = TimeSpan.FromMilliseconds(0);
				a3.SetValue(Storyboard.TargetPropertyProperty, new PropertyPath("Opacity"));
				Storyboard.SetTarget(a3, Target);
				s.Children.Add(a3);
			}
			//Visibility
			ObjectAnimationUsingKeyFrames key = new ObjectAnimationUsingKeyFrames();
			DiscreteObjectKeyFrame frame = new DiscreteObjectKeyFrame() { Value = state };
			if (state == Visibility.Visible || !CollapseHorizontal && !CollapseVertical && !CollapseOpacity)
				frame.KeyTime = TimeSpan.FromMilliseconds(0);
			else
				frame.KeyTime = this.Duration;
			key.KeyFrames.Add(frame);
			key.SetValue(Storyboard.TargetPropertyProperty, new PropertyPath("Visibility"));
			Storyboard.SetTarget(key, Target);
			s.Children.Add(key);

			return s;
		}


		public bool CollapseHorizontal
		{
			get { return (bool)GetValue(CollapseHorizontalProperty); }
			set { SetValue(CollapseHorizontalProperty, value); }
		}

		/// <summary>
		/// Identifies the <see cref="CollapseHorizontal"/> dependency property.
		/// </summary>
		public static readonly DependencyProperty CollapseHorizontalProperty =
			DependencyProperty.Register("CollapseHorizontal", typeof(bool), typeof(ToggleCollapseAction), new PropertyMetadata(true, OnPropertyChanged));

		public bool CollapseVertical
		{
			get { return (bool)GetValue(CollapseVerticalProperty); }
			set { SetValue(CollapseVerticalProperty, value); }
		}

		/// <summary>
		/// Identifies the <see cref="CollapseVertical"/> dependency property.
		/// </summary>
		public static readonly DependencyProperty CollapseVerticalProperty =
			DependencyProperty.Register("CollapseVertical", typeof(bool), typeof(ToggleCollapseAction), new PropertyMetadata(true, OnPropertyChanged));


		public bool CollapseOpacity
		{
			get { return (bool)GetValue(CollapseOpacityProperty); }
			set { SetValue(CollapseOpacityProperty, value); }
		}

		/// <summary>
		/// Identifies the <see cref="CollapseOpacity"/> dependency property.
		/// </summary>
		public static readonly DependencyProperty CollapseOpacityProperty =
			DependencyProperty.Register("CollapseOpacity", typeof(bool), typeof(ToggleCollapseAction), new PropertyMetadata(true, OnPropertyChanged));




		public TimeSpan Duration
		{
			get { return (TimeSpan)GetValue(DurationProperty); }
			set { SetValue(DurationProperty, value); }
		}

		/// <summary>
		/// Identifies the <see cref="Duration"/> dependency property.
		/// </summary>
		public static readonly DependencyProperty DurationProperty =
			DependencyProperty.Register("Duration", typeof(TimeSpan), typeof(ToggleCollapseAction), new PropertyMetadata(TimeSpan.FromMilliseconds(250), OnPropertyChanged));

		private static void OnPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			(d as ToggleCollapseAction).uncollapseStoryboard = null;
			(d as ToggleCollapseAction).collapseStoryboard = null;	
		}
	}
}
